@production_ohwr = {
  :name         => 'ohwr production',
  :application  => 'ohwr',
  :deploy_to    => '/var/rails/ohwr',
  :domain       => 'rails_ohwr',
  :rails_env    => 'production',
  :repository   => 'git://ohwr.org/ohr-support.git',
  :revision     => "remotes/origin/master",
  :ssh_flags    => '-p 22',
  :shared_paths => {
    'log'                                  => 'log',
    'system'                               => 'public/system',
    'pids'                                 => 'tmp/pids',
    'config/database.yml'                  => 'config/database.yml',
    'config/email.yml'                     => 'config/email.yml',
    'config/newrelic.yml'                  => 'config/newrelic.yml',
    'config/scm.yml'                       => 'config/scm.yml',
    'config/initializers/errbit.rb'        => 'config/initializers/erribt.rb',
    'config/initializers/session_store.rb' => 'config/initializers/session_store.rb',
    'files'                                => 'files',
    '.bundle'                              => '.bundle'
  }
}

@staging_ohwr = {
  :name         => 'ohr staging',
  :application  => 'ohwr.staging.es',
  :deploy_to    => '/var/rails/ohwr',
  :domain       => 'rails_alphastaging',
  :rails_env    => 'staging',
  :repository   => 'git://ohwr.org/ohr-support.git',
  :revision     => "remotes/origin/master",
  :shared_paths => {
    'log'                                  => 'log',
    'system'                               => 'public/system',
    'pids'                                 => 'tmp/pids',
    'config/database.yml'                  => 'config/database.yml',
    'config/email.yml'                     => 'config/email.yml',
    'config/newrelic.yml'                  => 'config/newrelic.yml',
    'config/scm.yml'                       => 'config/scm.yml',
    'config/initializers/errbit.rb'        => 'config/initializers/erribt.rb',
    'config/initializers/session_store.rb' => 'config/initializers/session_store.rb',
    'files'                                => 'files',
    '.bundle'                              => '.bundle'
  }
}

namespace :ohwr do

  desc "Points vlad to the production server"
  task :production do
    prepare(@production_ohwr)
    Rake::Task['vlad:confirm'].invoke
  end

  desc "Points vlad to the staging server"
  task :staging do
    @staging_ohwr[:revision] = "remotes/origin/#{ENV['BRANCH']}" unless ENV['BRANCH'].nil?
    prepare(@staging_ohwr)
  end

  namespace :production do
    desc "Updates production with the latest version"
    remote_task :deploy => %w[
      ohwr:production
      ohwr:private:deploy
    ]
    desc "Updates production without maintenance or migrations"
    remote_task :hot_deploy => %w[
      ohwr:production
      ohwr:private:hot_deploy
    ]

  end

  namespace :staging do
    desc "Updates staging with the latest version"
    remote_task :deploy => %w[
      ohwr:staging
      ohwr:private:deploy
    ]

    desc "Updates staging without maintenance or migrations"
    remote_task :hot_deploy => %w[
      ohwr:staging
      ohwr:private:hot_deploy
    ]

    desc "Clone production server on staging"
    remote_task :clone_production => %w[
      ohwr:staging
      vlad:maintenance:set
      ohwr:private:dump
      ohwr:private:clone_db
      ohwr:private:adapt_settings_for_staging
      ohwr:private:clone_files
      vlad:restart
      vlad:maintenance:remove
    ]
  end

  namespace :private do
    desc 'Clone DB from production to user staging'
    remote_task :clone_db => %w[ ohwr:staging ] do
      info "Clonning DB from production to staging"
      run "pg_dump -c -h #{@production_ohwr[:domain]} -U ohwr ohwr | psql -h localhost -U ohwr ohwr"
      remote_rake 'postgresql:remove_anonymous_role'
    end

    desc "Makes a clone of the production files directory on staging"
    remote_task :clone_files => %w[ ohwr:staging ] do
      info "Cloning production files into staging"
      files_path = "#{shared_path}/files"
      run "rm -rf #{files_path}"
      run "mkdir -p #{files_path}"
      run "rsync -r #{@production_ohwr[:domain]}:/#{files_path} #{files_path}"
    end

    desc "deploy to server" # code, files, config_files, db_migrate, clean_up
    task :deploy => %w[
      vlad:maintenance:set
      ohwr:private:dump
      vlad:update
      vlad:bundle:install
      ohwr:private:migrate
      ohwr:private:cleanup
      vlad:restart
      vlad:maintenance:remove
      vlad:notify_deploy
      vlad:cleanup
    ]

    desc "deploy to server without maintenance, bundle, or migrations"
    task :hot_deploy => %w[
      ohwr:private:dump
      vlad:update
      ohwr:private:cleanup
      vlad:restart
      vlad:cleanup
    ]

    desc "makes a security copy of the db and files folder on the dump folder"
    remote_task :dump => %w[
      ohwr:private:dump:db
      ohwr:private:dump:files
      ohwr:private:dump:cleanup
    ]

    desc "Executing migrations of database and plugins"
    remote_task :migrate do
      info "Executing rake db:migrate db:migrate:plugins"
      #remote_rake 'db:migrate:upgrade_plugin_migrations db:migrate_plugins db:migrate'
      remote_rake 'db:migrate db:migrate:plugins'
    end

    desc "Clears OHWR sessions and cache"
    remote_task :cleanup do
      info "OHWR clean up"
      remote_rake 'tmp:cache:clear tmp:sessions:clear'
    end

    desc "Set the values for checkout in staging"
    remote_task :adapt_settings_for_staging => %w[ ohwr:staging ] do
      info 'Executing OHWR Checkout & Gitolite'
      remote_rake 'staging:update_settings'
    end

    #namespace ohwr:private:dump
    namespace :dump do
      desc "dump ohwr database"
      remote_task :db do
        info "Dumping the database"
        run "pg_dump -c -U ohwr ohwr | gzip > #{File.join(dump_path, 'ohwr.sql.gz')}"
      end

      desc "compressed copy of files folder into the dump folder"
      remote_task :files do
        info "Dumping the files folder"
        run "tar -czf #{dump_path}/files.tgz #{shared_path}/files/"
      end

      desc "removes older releases from the dump folder"
      remote_task :cleanup do
        info "Cleaning up dump folder"
        max = keep_releases
        if dump_releases.length <= max then
          puts "no old dump releases to clean up #{dump_releases.length} <= #{max}"
        else
          puts "keeping #{max} of #{dump_releases.length} dump releases"

          directories = (dump_releases - dump_releases.last(max)).map { |release|
            File.join(dumps_path, release)
          }.join(" ")

          run "rm -rf #{directories}"
        end #if
      end # :cleanup

    end # namespace :dump
  end  #namespace :private
end
